/* instruction.h */

/* Copyright (C) 2006 Niels Mller
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef NXT_INSTRUCTION_H
#define NXT_INSTRUCTION_H

#include <stdio.h>

#include "symbol.h"
#include "keyword.h"

struct program;

enum instruction_type
  {
    instruction_binary, instruction_finclump,
  };

struct instruction
{
  struct instruction *next;
  struct instruction *prev;

  unsigned lineno;

  /* Both address and size is counted in units of 16-bit words. */
  unsigned address;
  unsigned size;

  /* The operation */
  enum keyword operation;

  enum instruction_type type;  
};

struct instruction *
instruction_binary_create (struct program *program, enum keyword op,
			   const struct symbol *dst,
			   const struct symbol *s0, const struct symbol *s1, unsigned lineno);

struct instruction *
instruction_finclump_create (unsigned lineno);

int
instruction_output (struct instruction *self,
		    struct environment *local,
		    struct environment *global,
		    FILE *f);

#endif /* NXT_INSTRUCTION_H */
