/* nxt-as.c
 *
 * Main assembler program. */

/* Copyright (C) 2006 Niels Mller
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#if HAVE_CONFIG_H
# include "config.h"
#endif

#include <errno.h>
#include <stdlib.h>
#include <string.h>

#include "instruction.h"
#include "program.h"
#include "record.h"
#include "symbol.h"

#define DEFAULT_OUTUT_FILE "a.rxe"

static void
display_binding (struct binding *binding, void *ctx UNUSED)
{
  switch (binding->type)
    {
    case binding_label:
      fprintf (stderr, "%10s:    %04x L\n",
	       binding->symbol->name, binding->definition.instruction->address);
      break;

    case binding_data:
      fprintf (stderr, "%10s: %2d %04x D\n",
	       binding->symbol->name,
	       binding->definition.record->index,
	       binding->definition.record->address);
      break;

    case binding_clump:
      fprintf (stderr, "%10s:          C\n", binding->symbol->name);
      break;

    default:
      abort();
    }
}

static void
display_bindings (struct program *program)
{
  struct clump *p;
      
  fprintf(stderr, "Globals:\n");
  environment_foreach (program->global, display_binding, NULL);

  for (p = program->clump_start; p; p = p->next)
    {
      fprintf(stderr, "New clump:\n");
      environment_foreach (p->local, display_binding, NULL);
    }
}

int
main(int argc, char **argv)
{
  struct program *program;
  FILE *f;

  if (argc != 2)
    fprintf (stderr, "Usage: nxt-as INPUT-FILE\n");

  f = fopen (argv[1], "r");
  if (!f)
    {
      fprintf(stderr, "Could not open input file '%s': %s.\n",
	      argv[1], strerror(errno));
      return EXIT_FAILURE;
    }

  program = program_parse_file (argv[1], f);

  fclose (f);
  
  if (!program)
    return EXIT_FAILURE;

  program_allocate_data (program);
  program_allocate_instructions (program);

  display_bindings (program);
  
  f = fopen (DEFAULT_OUTUT_FILE, "wb");
  if (!f)
    {
      fprintf(stderr, "Could not open output file '%s': %s.\n",
	      DEFAULT_OUTUT_FILE, strerror(errno));
      return EXIT_FAILURE;
    }
  
  if (program_output (program, f) && fclose (f) == 0)
    return EXIT_SUCCESS;
  else
    {
      fprintf(stderr, "Writing output file failed: %s\n", strerror (errno));
      return EXIT_FAILURE;
    }
}
