/* program.h */

/* Copyright (C) 2006 Niels Mller
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef NXT_PROGRAM_H
#define NXT_PROGRAM_H

struct instruction;
struct symbol;

struct clump
{
  struct clump *next;
  struct clump *prev;

  struct environment *local;

  struct instruction *code_start;
  struct instruction *code_end;

  const struct symbol *start;

  /* FIXME: Initial count, and dependants */
};

struct program
{
  struct symbol_table *symbols;
  struct environment *global;

  /* The data instructions */
  struct record *data_start;
  struct record *data_end;

  struct clump *clump_start;
  struct clump *clump_end;

  /* Number of entries in the dataspace table of contents */
  unsigned data_count;
  unsigned static_data_size;
  unsigned static_initializer_size;

  unsigned clump_count;

  /* In units of 16-bit words */
  unsigned codeword_count;
};

struct clump *
clump_create (void);

struct program *
program_create (void);

struct clump *
program_start_clump (struct program *self,
		     const struct symbol *start, unsigned lineno);

int
program_end_clump (struct program *self);

int
program_add_data (struct program *self,
		  const struct symbol *symbol, struct record *record,
		  unsigned lineno);

void
program_add_instruction (struct program *self,
			 struct instruction *instruction);

int
program_define_label (struct program *self, const struct symbol *label,
		      struct instruction *instruction, unsigned lineno);

/* Main passes */
struct program *
program_parse_file (const char *filename, FILE *f);

void
program_allocate_data (struct program *self);

void
program_allocate_instructions (struct program *self);

int
program_output (struct program *self, FILE *f);

#endif /* NXT_PROGRAM_H */
