/* symbol.h */

/* Copyright (C) 2006 Niels Mller
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef NXT_SYMBOL_H
#define NXT_SYMBOL_H

struct instruction;

struct symbol_table;

struct symbol_table *
symbol_table_create (void);

struct symbol
{
  unsigned name_length;
  const char *name;

  unsigned first_seen_lineno;

  /* For the symbol hash tables */
  unsigned hash_value;
  const struct symbol *next;  
};

const struct symbol *
symbol_intern (struct symbol_table *self,
	       const char *name, unsigned name_length,
	       unsigned lineno);

struct environment;

struct environment *
environment_create (void);

enum binding_type { binding_label, binding_data, binding_clump };

struct binding
{
  const struct symbol *symbol;
  
  unsigned first_seen_lineno;

  enum binding_type type;

  union {
    struct instruction *instruction;
    struct record *record;
    struct clump *clump;
  } definition;

  unsigned definition_lineno;

  struct binding *next;
};

struct binding *
environment_resolve (struct environment *self,
		     const struct symbol *symbol,
		     enum binding_type type,
		     unsigned lineno);

struct instruction *
environment_resolve_label (struct environment *self,
			   const struct symbol *symbol);

struct record *
environment_resolve_record (struct environment *self,
			    struct environment *global,
			    const struct symbol *symbol);

int
environment_bind_label (struct environment *self,
			const struct symbol *symbol,
			struct instruction *definition, unsigned lineno);

int
environment_bind_data (struct environment *self,
		       const struct symbol *symbol,
		       struct record *definition, unsigned lineno);

int
environment_bind_clump (struct environment *self,
			const struct symbol *symbol,
			struct clump *definition, unsigned lineno);

int
environment_handle_undefined (struct environment *self, struct environment *other);

void
environment_foreach (struct environment *self,
		     void (*f)(struct binding *, void *),
		     void *ctx);

#endif /* NXT_SYMBOL_H */
